using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace WindowsGame1
{
    /// <summary>
    /// 基底 Game クラスから派生した、ゲームのメイン クラスです。
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        //複数の出力先を用意する
        RenderTarget2D rTarget1;
        RenderTarget2D rTarget2;
        RenderTargetBinding[] rTargets;

        //適用するシェーダとテクスチャ
        Effect effect;
        Texture2D tex2d_Lena;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            this.graphics.PreferredBackBufferWidth = 1024;
            this.graphics.PreferredBackBufferHeight = 512;
        }

        /// <summary>
        /// ゲームが実行を開始する前に必要な初期化を行います。
        /// ここで、必要なサービスを照会して、関連するグラフィック以外のコンテンツを
        /// 読み込むことができます。base.Initialize を呼び出すと、使用するすべての
        /// コンポーネントが列挙されるとともに、初期化されます。
        /// </summary>
        protected override void Initialize()
        {
            // TODO: ここに初期化ロジックを追加します。

            base.Initialize();
        }

        /// <summary>
        /// LoadContent はゲームごとに 1 回呼び出され、ここですべてのコンテンツを
        /// 読み込みます。
        /// </summary>
        protected override void LoadContent()
        {
            // 新規の SpriteBatch を作成します。これはテクスチャーの描画に使用できます。
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: this.Content クラスを使用して、ゲームのコンテンツを読み込みます。
            
            effect = Content.Load<Effect>("Effect1");
            tex2d_Lena = Content.Load<Texture2D>("lena");

            //出力先を生成します
            rTarget1 = new RenderTarget2D(GraphicsDevice, tex2d_Lena.Width,
                tex2d_Lena.Height, false, SurfaceFormat.Color, DepthFormat.Depth24);

            //2つ目の出力先
            rTarget2 = new RenderTarget2D(GraphicsDevice, tex2d_Lena.Width,
                tex2d_Lena.Height, false, SurfaceFormat.Color, DepthFormat.Depth24);

            //出力先情報をまとめて管理することができます
            rTargets = new RenderTargetBinding[2]
            {
                rTarget1,rTarget2
            };

        }

        /// <summary>
        /// UnloadContent はゲームごとに 1 回呼び出され、ここですべてのコンテンツを
        /// アンロードします。
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: ここで ContentManager 以外のすべてのコンテンツをアンロードします。
        }

        /// <summary>
        /// ワールドの更新、衝突判定、入力値の取得、オーディオの再生などの
        /// ゲーム ロジックを、実行します。
        /// </summary>
        /// <param name="gameTime">ゲームの瞬間的なタイミング情報</param>
        protected override void Update(GameTime gameTime)
        {
            // ゲームの終了条件をチェックします。
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: ここにゲームのアップデート ロジックを追加します。

            base.Update(gameTime);
        }

        /// <summary>
        /// ゲームが自身を描画するためのメソッドです。
        /// </summary>
        /// <param name="gameTime">ゲームの瞬間的なタイミング情報</param>
        protected override void Draw(GameTime gameTime)
        {
            // TODO: ここに描画コードを追加します。

            //描画結果の出力先を、複数のRenderTargetをまとめた物にします
            GraphicsDevice.SetRenderTargets(rTargets);
            GraphicsDevice.Clear(Color.CornflowerBlue);

            //Effectを使って、設定した複数のRenderTargetにテクスチャをそれぞれ描画します
            spriteBatch.Begin(SpriteSortMode.Deferred,null,null,null,null,effect);
            spriteBatch.Draw(tex2d_Lena,Vector2.Zero,Color.White);
            spriteBatch.End();

            //Rendartargetをスクリーンに戻します
            //≒描画結果の出力先をスクリーンにします。
            GraphicsDevice.SetRenderTargets(null);
            GraphicsDevice.Clear(Color.CornflowerBlue);

            //先にEffectを使って描画した結果をスクリーン上に描画します。
            spriteBatch.Begin();
            spriteBatch.Draw(rTarget1, Vector2.Zero, Color.White);
            spriteBatch.Draw(rTarget2,new Vector2(tex2d_Lena.Width,0),Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
